"""
The module curves exports functions to approximate algebraic space curves
with rational expressions, in particular Pade approximants.
"""

def get_homotopy_continuation_parameter(idx):
    """
    Returns the current value of the homotopy continuation parameter
    with index idx, where idx is an integer in the interval [1, 12].
    """
    from phcpy.phcpy2c2 import py2c_padcon_get_homotopy_continuation_parameter
    return py2c_padcon_get_homotopy_continuation_parameter(idx)

def get_gamma_constant():
    """
    Returns the current value of the gamma constant in the homotopy.
    A random value for gamma will guarantee the absence of singular
    solutions along a path, as unlucky choices belong to an algebraic set.
    A tuple of two floats is returned, respectively with the real
    and imaginary parts of the complex value for the gamma constant.
    """
    return get_homotopy_continuation_parameter(1)

def get_degree_of_numerator():
    """
    Returns the current value of the degree of the numerator of the
    Pade approximant, evaluated to predict the next solution on a path.
    """
    return get_homotopy_continuation_parameter(2)

def get_degree_of_denominator():
    """
    Returns the current value of the degree of the denominator of the
    Pade approximant, evaluated to predict the next solution on a path.
    """
    return get_homotopy_continuation_parameter(3)

def get_maximum_step_size():
    """
    Returns the current value of the maximum step size.
    The step size is the increment to the continuation parameter.
    """
    return get_homotopy_continuation_parameter(4)

def get_minimum_step_size():
    """
    Returns the current value of the minimum step size.
    The path tracking will stop if the step size is larger than the
    minimum step size and if the predictor residual is larger than
    the value for alpha, the tolerance on the predictor residual.
    """
    return get_homotopy_continuation_parameter(5)

def get_series_beta_factor():
    """
    Returns the current multiplication factor of the step size set
    by the power series approximation for the algebraic curve.
    """
    return get_homotopy_continuation_parameter(6)

def get_pole_radius_beta_factor():
    """
    Returns the current multiplication factor of the smallest pole radius.
    The smallest radius of the poles of the Pade approximant gives
    an upper bound on a safe step size.  The step size is set by
    multiplication of the smallest pole radius with the beta factor. 
    """
    return get_homotopy_continuation_parameter(7)

def get_curvature_beta_factor():
    """
    Returns the current multiplication factor of the curvature bound.
    This curvature bound gives an upper bound on a safe step size.
    The step size is set by multiplication of the curvature bound
    with the beta factor. 
    """
    return get_homotopy_continuation_parameter(8)

def get_predictor_residual_alpha():
    """
    Returns the current tolerance on the residual of the predictor.
    This alpha parameter controls the accuracy of the tracking.
    As long as the residual of the evaluated predicted solution
    is larger than alpha, the step size is cut in half.
    """
    return get_homotopy_continuation_parameter(9)

def get_corrector_residual_tolerance():
    """
    Returns the current tolerance on the corrector residual.
    The corrector stops if the residual of the current approximation
    drops below this tolerance.
    """
    return get_homotopy_continuation_parameter(10)

def get_zero_series_coefficient_tolerance():
    """
    Returns the current tolerance on the series coefficient to be zero.
    A coefficient in a power series will be considered as zero if
    its absolute value drops below this tolerance.
    """
    return get_homotopy_continuation_parameter(11)

def get_maximum_corrector_steps():
    """
    Returns the current value of the maximum number of corrector steps
    executed after the predictor stage.
    """
    return get_homotopy_continuation_parameter(12)

def get_maximum_steps_on_path():
    """
    Returns the current value of the maximum number of steps on a path.
    The path trackers abandons the tracking of a path once the number
    of steps reaches this maximum number.
    """
    return get_homotopy_continuation_parameter(13)

def set_homotopy_continuation_parameter(idx, val):
    """
    Sets the value of the homotopy continuation parameter with index idx,
    where idx is in an integer in range(2, 13), to the value val.
    """
    from phcpy.phcpy2c2 import py2c_padcon_set_homotopy_continuation_parameter
    return py2c_padcon_set_homotopy_continuation_parameter(idx, val)

def set_homotopy_continuation_gamma(regamma=0, imgamma=0):
    """
    Sets the value of the homotopy continuation gamma constant 
    to the complex number with real part in regamma
    and the imaginary part in imgamma.
    If both regamma and imgamma are zero,
    then the user is prompted to provide values for regamma and imgamma.
    """
    from phcpy.phcpy2c2 import py2c_padcon_set_homotopy_continuation_gamma
    if((regamma == 0) and (imgamma == 0)):
        regm = float(raw_input('-> give the real part of gamma : '))
        imgm = float(raw_input('-> give the imaginary part of gamma : '))
        return py2c_padcon_set_homotopy_continuation_gamma(regm, imgm)
    else:
        return py2c_padcon_set_homotopy_continuation_gamma(regamma, imgamma)

def write_homotopy_continuation_parameters():
    """
    Writes the values of the homotopy continuation parameters.
    """
    pars = [get_homotopy_continuation_parameter(k) for k in range(1, 13)]
    regamma, imgamma = pars[0]
    print "Values of the HOMOTOPY CONTINUATION PARAMETERS :"
    print " 1. gamma :", regamma + imgamma*complex(0,1)
    print " 2. degree of numerator of Pade approximant    :", pars[1]
    print " 3. degree of denominator of Pade approximant  :", pars[2]
    print " 4. maximum step size                          :", pars[3]
    print " 5. minimum step size                          :", pars[4]
    print " 6. multiplication factor for the pole radius  :", pars[6]
    print " 7. multiplication factor for the curvature    :", pars[6]
    print " 8. tolerance on the residual of the predictor :", pars[7]
    print " 9. tolerance on the residual of the corrector :", pars[8]
    print "10. tolerance on zero series coefficients      :", pars[9]
    print "11. maximum number of corrector steps          :", pars[10]
    print "12. maximum steps on a path                    :", pars[11]

def tune_homotopy_continuation_parameters():
    """
    The interactive loop prompts the user to tune the parameters.
    """
    idx = 1
    while(idx > 0):
        write_homotopy_continuation_parameters()
        idxprompt = 'To change a value, give an index (0 to exit) : '
        idx = int(raw_input(idxprompt))
        if(idx > 0 and idx < 13):
            if(idx == 1):
                set_homotopy_continuation_gamma()
            else:
                valprompt = '-> give a value for parameter %d : ' % idx
                val = float(raw_input(valprompt))
                set_homotopy_continuation_parameter(idx, val);

def reset_homotopy_continuation_parameters(precision=0):
    """
    Resets the homotopy continuation parameters for the step-by-step
    path trackers, using the value of the precision, 0 for double,
    1 for double double, or 2 for quad double.
    """
    from phcpy.phcpy2c2 \
        import py2c_padcon_reset_homotopy_continuation_parameters
    return py2c_padcon_reset_homotopy_continuation_parameters(precision)

def standard_track\
    (target, start, sols, filename="", verbose=False, mhom=0, partition=None):
    r"""
    Wraps the tracker for Pade continuation in standard double precision.
    On input are a target system, a start system with solutions,
    optionally: a string *filename* and the *verbose* flag.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    By default *mhom* is zero and tracking happens in the original coordinates,
    if *mhom* equals one, then 1-homogeneous coordinates are used, and
    if *mhom* is two or higher, then multi-homogenization applies and
    *partition* contains the index representation of the partition of
    the set of variables.  This index representation is a list of as many
    indices as the number of variables, defining which set of the partition
    each variables belongs to.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_create_standard_homotopy_with_gamma
    from phcpy.phcpy2c2 import py2c_clear_standard_homotopy
    from phcpy.phcpy2c2 import py2c_clear_standard_operations_data
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    from phcpy.solver import number_of_symbols
    from phcpy.phcpy2c2 import py2c_padcon_standard_track
    dim = number_of_symbols(start)
    store_standard_system(target, nbvar=dim)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start, nbvar=dim)
    py2c_copy_standard_container_to_start_system()
    store_standard_solutions(dim, sols)
    py2c_copy_standard_container_to_start_solutions()
    (regamma, imgamma) = get_homotopy_continuation_parameter(1)
    py2c_create_standard_homotopy_with_gamma(regamma, imgamma)
    nbc = len(filename)
    fail = py2c_padcon_standard_track\
               (nbc,filename,0,int(verbose),mhom,dim,str(partition))
    # py2c_clear_standard_homotopy()
    # py2c_clear_standard_operations_data()
    return load_standard_solutions()

def dobldobl_track\
    (target, start, sols, filename="", verbose=False, mhom=0, partition=None):
    r"""
    Wraps the tracker for Pade continuation in double double precision.
    On input are a target system, a start system with solutions,
    optionally: a string *filename* and the *verbose* flag.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    By default *mhom* is zero and tracking happens in the original coordinates,
    if *mhom* equals one, then 1-homogeneous coordinates are used, and
    if *mhom* is two or higher, then multi-homogenization applies and
    *partition* contains the index representation of the partition of
    the set of variables.  This index representation is a list of as many
    indices as the number of variables, defining which set of the partition
    each variables belongs to.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_create_dobldobl_homotopy_with_gamma
    from phcpy.phcpy2c2 import py2c_clear_dobldobl_homotopy
    from phcpy.phcpy2c2 import py2c_clear_dobldobl_operations_data
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    from phcpy.solver import number_of_symbols
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_track
    dim = number_of_symbols(start)
    store_dobldobl_system(target, nbvar=dim)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start, nbvar=dim)
    py2c_copy_dobldobl_container_to_start_system()
    (regamma, imgamma) = get_homotopy_continuation_parameter(1)
    store_dobldobl_solutions(dim, sols)
    py2c_copy_dobldobl_container_to_start_solutions()
    nbc = len(filename)
    (regamma, imgamma) = get_homotopy_continuation_parameter(1)
    py2c_create_dobldobl_homotopy_with_gamma(regamma, imgamma)
    fail = py2c_padcon_dobldobl_track\
               (nbc,filename,0,int(verbose),mhom,dim,str(partition))
    # py2c_clear_dobldobl_homotopy()
    # py2c_clear_dobldobl_operations_data()
    return load_dobldobl_solutions()

def quaddobl_track\
    (target, start, sols, filename="", verbose=False, mhom=0, partition=None):
    r"""
    Wraps the tracker for Pade continuation in quad double precision.
    On input are a target system, a start system with solutions,
    optionally: a string *filename* and the *verbose* flag.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_create_quaddobl_homotopy_with_gamma
    from phcpy.phcpy2c2 import py2c_clear_quaddobl_homotopy
    from phcpy.phcpy2c2 import py2c_clear_quaddobl_operations_data
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    from phcpy.solver import number_of_symbols
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_track
    dim = number_of_symbols(start)
    store_quaddobl_system(target, nbvar=dim)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start, nbvar=dim)
    py2c_copy_quaddobl_container_to_start_system()
    (regamma, imgamma) = get_homotopy_continuation_parameter(1)
    store_quaddobl_solutions(dim, sols)
    py2c_copy_quaddobl_container_to_start_solutions()
    (regamma, imgamma) = get_homotopy_continuation_parameter(1)
    py2c_create_quaddobl_homotopy_with_gamma(regamma, imgamma)
    nbc = len(filename)
    fail = py2c_padcon_quaddobl_track\
               (nbc,filename,0,int(verbose),mhom,dim,str(partition))
    # py2c_clear_quaddobl_homotopy()
    # py2c_clear_quaddobl_operations_data()
    return load_quaddobl_solutions()

def standard_set_homotopy(target, start, verbose=False, homogeneous=False):
    """
    Initializes the homotopy with the target and start system for a
    step-by-step run of the series-Pade tracker, in double precision.
    If verbose, then extra output is written.
    If homogeneous, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_system
    from phcpy.interface import store_standard_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_standard_system(target, nbvar=dim)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start, nbvar=dim)
    py2c_copy_standard_container_to_start_system()
    from phcpy.phcpy2c2 import py2c_padcon_standard_initialize_homotopy
    vrb, hmg = int(verbose), int(homogeneous)
    return py2c_padcon_standard_initialize_homotopy(vrb, hmg)

def dobldobl_set_homotopy(target, start, verbose=False, homogeneous=False):
    """
    Initializes the homotopy with the target and start system for a
    step-by-step run of the series-Pade tracker, in double double precision.
    If verbose, then extra output is written.
    If homogeneous, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.interface import store_dobldobl_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_dobldobl_system(target, nbvar=dim)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start, nbvar=dim)
    py2c_copy_dobldobl_container_to_start_system()
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_initialize_homotopy
    vrb, hmg = int(verbose), int(homogeneous)
    return py2c_padcon_dobldobl_initialize_homotopy(vrb, hmg)

def quaddobl_set_homotopy(target, start, verbose=False, homogeneous=False):
    """
    Initializes the homotopy with the target and start system for a
    step-by-step run of the series-Pade tracker, in quad double precision.
    If verbose, then extra output is written.
    If homogeneous, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.interface import store_quaddobl_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_quaddobl_system(target, nbvar=dim)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start, nbvar=dim)
    py2c_copy_quaddobl_container_to_start_system()
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_initialize_homotopy
    vrb, hmg = int(verbose), int(homogeneous)
    return py2c_padcon_quaddobl_initialize_homotopy(vrb, hmg)

def standard_set_parameter_homotopy(hom, idx, verbose=False):
    """
    Initializes the homotopy with the polynomials in hom for a
    step-by-step run of the series-Pade tracker, in double precision.
    The value idx gives the index of the continuation parameter
    and is the index of one of the variables in the homotopy hom.
    If verbose, then extra output is written.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_target_system
    from phcpy.interface import store_standard_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(hom)
    store_standard_system(hom, nbvar=dim)
    py2c_copy_standard_container_to_target_system()
    from phcpy.phcpy2c2  \
        import py2c_padcon_standard_initialize_parameter_homotopy
    vrb = int(verbose)
    return py2c_padcon_standard_initialize_parameter_homotopy(idx, vrb)

def dobldobl_set_parameter_homotopy(hom, idx, verbose=False):
    """
    Initializes the homotopy with the polynomials in hom for a
    step-by-step run of the series-Pade tracker, in double double precision.
    The value idx gives the index of the continuation parameter
    and is the index of one of the variables in the homotopy hom.
    If verbose, then extra output is written.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.interface import store_dobldobl_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(hom)
    store_dobldobl_system(hom, nbvar=dim)
    py2c_copy_dobldobl_container_to_target_system()
    from phcpy.phcpy2c2  \
        import py2c_padcon_dobldobl_initialize_parameter_homotopy
    vrb = int(verbose)
    return py2c_padcon_dobldobl_initialize_parameter_homotopy(idx, vrb)

def quaddobl_set_parameter_homotopy(hom, idx, verbose=False):
    """
    Initializes the homotopy with the polynomials in hom for a
    step-by-step run of the series-Pade tracker, in quad double precision.
    The value idx gives the index of the continuation parameter
    and is the index of one of the variables in the homotopy hom.
    If verbose, then extra output is written.
    Returns the failure code of the homotopy initializer.
    """
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.interface import store_quaddobl_system
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(hom)
    store_quaddobl_system(hom, nbvar=dim)
    py2c_copy_quaddobl_container_to_target_system()
    from phcpy.phcpy2c2  \
        import py2c_padcon_quaddobl_initialize_parameter_homotopy
    vrb = int(verbose)
    return py2c_padcon_quaddobl_initialize_parameter_homotopy(idx, vrb)

def standard_set_solution(nvar, sol, verbose=False):
    r"""
    Sets the start solution in *sol* for the step-by-step run of
    the series-Pade tracker, in standard double precision.
    If verbose, then extra output is written.
    The number of variables is in *nvar*.
    """
    from phcpy.phcpy2c2 import py2c_padcon_initialize_standard_solution
    from phcpy.interface import store_standard_solutions
    store_standard_solutions(nvar, [sol])
    return py2c_padcon_initialize_standard_solution(1, int(verbose))

def dobldobl_set_solution(nvar, sol, verbose=False):
    r"""
    Sets the start solution in *sol* for the step-by-step run of
    the series-Pade tracker, in double double precision.
    If verbose, then extra output is written.
    The number of variables is in *nvar*.
    """
    from phcpy.phcpy2c2 import py2c_padcon_initialize_dobldobl_solution
    from phcpy.interface import store_dobldobl_solutions
    store_dobldobl_solutions(nvar, [sol])
    return py2c_padcon_initialize_dobldobl_solution(1, int(verbose))

def quaddobl_set_solution(nvar, sol, verbose=False):
    r"""
    Sets the start solution in *sol* for the step-by-step run of
    the series-Pade tracker, in quad double precision.
    If verbose, then extra output is written.
    The number of variables is in *nvar*.
    """
    from phcpy.phcpy2c2 import py2c_padcon_initialize_quaddobl_solution
    from phcpy.interface import store_quaddobl_solutions
    store_quaddobl_solutions(nvar, [sol])
    return py2c_padcon_initialize_quaddobl_solution(1, int(verbose))

def standard_get_solution(verbose=False):
    """
    Returns the current solution on the path, in double precision,
    which starts at the solution set with standard_set_solution().
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_get_standard_solution
    from phcpy.phcpy2c2 import py2c_solcon_length_standard_solution_string
    from phcpy.phcpy2c2 import py2c_solcon_write_standard_solution_string
    py2c_padcon_get_standard_solution(1, int(verbose))
    lns = py2c_solcon_length_standard_solution_string(1)
    return py2c_solcon_write_standard_solution_string(1, lns)

def dobldobl_get_solution(verbose=False):
    """
    Returns the current solution on the path, in double double precision,
    which starts at the solution set with dobldobl_set_solution().
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_get_dobldobl_solution
    from phcpy.phcpy2c2 import py2c_solcon_length_dobldobl_solution_string
    from phcpy.phcpy2c2 import py2c_solcon_write_dobldobl_solution_string
    py2c_padcon_get_dobldobl_solution(1, int(verbose))
    lns = py2c_solcon_length_dobldobl_solution_string(1)
    return py2c_solcon_write_dobldobl_solution_string(1, lns)

def quaddobl_get_solution(verbose=False):
    """
    Returns the current solution on the path, in quad double precision,
    which starts at the solution set with quaddobl_set_solution().
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_get_quaddobl_solution
    from phcpy.phcpy2c2 import py2c_solcon_length_quaddobl_solution_string
    from phcpy.phcpy2c2 import py2c_solcon_write_quaddobl_solution_string
    py2c_padcon_get_quaddobl_solution(1, int(verbose))
    lns = py2c_solcon_length_quaddobl_solution_string(1)
    return py2c_solcon_write_quaddobl_solution_string(1, lns)

def standard_predict_correct(verbose=False):
    """
    Performs one predictor and one corrector step on the set homotopy
    and the set solution, in standard double precision.
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_predict_correct
    py2c_padcon_standard_predict_correct(int(verbose))

def dobldobl_predict_correct(verbose=False):
    """
    Performs one predictor and one corrector step on the set homotopy
    and the set solution, in double double precision.
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_predict_correct
    py2c_padcon_dobldobl_predict_correct(int(verbose))

def quaddobl_predict_correct(verbose=False):
    """
    Performs one predictor and one corrector step on the set homotopy
    and the set solution, in quad double precision.
    If verbose, then extra output is written.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_predict_correct
    py2c_padcon_quaddobl_predict_correct(int(verbose))

def standard_t_value():
    """
    Returns the current t value in the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_t_value
    return py2c_padcon_standard_t_value()

def dobldobl_t_value():
    """
    Returns the current t value in the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_t_value
    return py2c_padcon_dobldobl_t_value()

def quaddobl_t_value():
    """
    Returns the current t value in the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_t_value
    return py2c_padcon_quaddobl_t_value()

def standard_step_size():
    """
    Returns the current step size in the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_step_size
    return py2c_padcon_standard_step_size()

def dobldobl_step_size():
    """
    Returns the current step size in the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_step_size
    return py2c_padcon_dobldobl_step_size()

def quaddobl_step_size():
    """
    Returns the current step size in the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_step_size
    return py2c_padcon_quaddobl_step_size()

def standard_series_step():
    """
    Returns the current series step in the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_series_step
    return py2c_padcon_standard_series_step()

def dobldobl_series_step():
    """
    Returns the current series step in the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_series_step
    return py2c_padcon_dobldobl_series_step()

def quaddobl_series_step():
    """
    Returns the current series step in the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_series_step
    return py2c_padcon_quaddobl_series_step()

def standard_pole_step():
    """
    Returns the current pole step in the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_pole_step
    return py2c_padcon_standard_pole_step()

def dobldobl_pole_step():
    """
    Returns the current pole step in the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_pole_step
    return py2c_padcon_dobldobl_pole_step()

def quaddobl_pole_step():
    """
    Returns the current pole step in the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_pole_step
    return py2c_padcon_quaddobl_pole_step()

def standard_estimated_distance():
    """
    Returns the current estimated distance to the closest solution computed
    by the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_estimated_distance
    return py2c_padcon_standard_estimated_distance()

def dobldobl_estimated_distance():
    """
    Returns the current estimated distance to the closest solution computed
    by the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_estimated_distance
    return py2c_padcon_dobldobl_estimated_distance()

def quaddobl_estimated_distance():
    """
    Returns the current estimated distance to the closest solution computed
    by the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_estimated_distance
    return py2c_padcon_quaddobl_estimated_distance()

def standard_hessian_step():
    """
    Returns the current Hessian step in the tracker in double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_hessian_step
    return py2c_padcon_standard_hessian_step()

def dobldobl_hessian_step():
    """
    Returns the current Hessian step in the tracker in double double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_hessian_step
    return py2c_padcon_dobldobl_hessian_step()

def quaddobl_hessian_step():
    """
    Returns the current Hessian step in the tracker in quad double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_hessian_step
    return py2c_padcon_quaddobl_hessian_step()

def standard_pole_radius():
    """
    Returns the smallest pole radius,
    used in the predictor in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_pole_radius
    return py2c_padcon_standard_pole_radius()

def dobldobl_pole_radius():
    """
    Returns the smallest pole radius,
    used in the predictor in double double precision.
    The returned double is the high part of the quad double.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_pole_radius
    return py2c_padcon_dobldobl_pole_radius()

def quaddobl_pole_radius():
    """
    Returns the smallest pole radius,
    used in the predictor in quad double precision.
    The returned double is the highest part of the quad double.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_pole_radius
    return py2c_padcon_quaddobl_pole_radius()

def standard_closest_pole():
    """
    Returns a tuple with the real and imaginary part of the closest pole
    used in the predictor in standard double precision.
    The result is only meaningful is the real part is positive.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_closest_pole
    return py2c_padcon_standard_closest_pole()

def dobldobl_closest_pole():
    """
    Returns a tuple with the real and imaginary part of the closest pole
    used in the predictor in double double precision.
    The result is only meaningful is the real part is positive.
    The double coefficients are the high parts of the double doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_closest_pole
    return py2c_padcon_dobldobl_closest_pole()

def quaddobl_closest_pole():
    """
    Returns a tuple with the real and imaginary part of the closest pole
    used in the predictor in quad double precision.
    The result is only meaningful is the real part is positive.
    The double coefficients are the highest parts of the quad doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_closest_pole
    return py2c_padcon_quaddobl_closest_pole()

def standard_series_coefficients(dim):
    """
    Returns a list of lists with the coefficients of the series
    computed by the predictor in standard double precision.
    On entry in dim is the number of variables.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_series_coefficient
    deg = get_degree_of_numerator() + get_degree_of_denominator()
    result = []
    for row in range(1, dim+1):
        cfs = []
        for col in range(deg+1):
            (rcf, icf) = py2c_padcon_standard_series_coefficient(row, col, 0)
            cfs.append(complex(rcf, icf))
        result.append(cfs)
    return result

def dobldobl_series_coefficients(dim):
    """
    Returns a list of lists with the coefficients of the series
    computed by the predictor in double double precision.
    On entry in dim is the number of variables.
    The double coefficients are the high parts of the double doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_series_coefficient
    deg = get_degree_of_numerator() + get_degree_of_denominator()
    result = []
    for row in range(1, dim+1):
        cfs = []
        for col in range(deg+1):
            (rcf, icf) = py2c_padcon_dobldobl_series_coefficient(row, col, 0)
            cfs.append(complex(rcf, icf))
        result.append(cfs)
    return result

def quaddobl_series_coefficients(dim):
    """
    Returns a list of lists with the coefficients of the series
    computed by the predictor in quad double precision.
    On entry in dim is the number of variables.
    The double coefficients are the highest parts of the quad doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_series_coefficient
    deg = get_degree_of_numerator() + get_degree_of_denominator()
    result = []
    for row in range(1, dim+1):
        cfs = []
        for col in range(deg+1):
            (rcf, icf) = py2c_padcon_quaddobl_series_coefficient(row, col, 0)
            cfs.append(complex(rcf, icf))
        result.append(cfs)
    return result

def standard_pade_coefficients(idx):
    """
    Returns a tuple of lists with the coefficients of the Pade approximants
    computed by the predictor in standard double precision.
    The first list in the tuple holds the coefficients of the numerator,
    the second list in the tuple holds the denominator coefficients.
    On entry in idx is the index of a variable.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_numerator_coefficient
    from phcpy.phcpy2c2 import py2c_padcon_standard_denominator_coefficient
    numdeg = get_degree_of_numerator()
    dendeg = get_degree_of_denominator()
    numcfs = []
    for col in range(numdeg+1):
        (rcf, icf) = py2c_padcon_standard_numerator_coefficient(idx, col, 0)
        numcfs.append(complex(rcf, icf))
    dencfs = []
    for col in range(dendeg+1):
        (rcf, icf) = py2c_padcon_standard_denominator_coefficient(idx, col, 0)
        dencfs.append(complex(rcf, icf))
    return (numcfs, dencfs)

def dobldobl_pade_coefficients(idx):
    """
    Returns a tuple of lists with the coefficients of the Pade approximants
    computed by the predictor in double double precision.
    The double coefficients are the high parts of the double doubles.
    The first list in the tuple holds the coefficients of the numerator,
    the second list in the tuple holds the denominator coefficients.
    On entry in idx is the index of a variable.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_numerator_coefficient
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_denominator_coefficient
    numdeg = get_degree_of_numerator()
    dendeg = get_degree_of_denominator()
    numcfs = []
    for col in range(numdeg+1):
        (rcf, icf) = py2c_padcon_dobldobl_numerator_coefficient(idx, col, 0)
        numcfs.append(complex(rcf, icf))
    dencfs = []
    for col in range(dendeg+1):
        (rcf, icf) = py2c_padcon_dobldobl_denominator_coefficient(idx, col, 0)
        dencfs.append(complex(rcf, icf))
    return (numcfs, dencfs)

def quaddobl_pade_coefficients(idx):
    """
    Returns a tuple of lists with the coefficients of the Pade approximants
    computed by the predictor in quad double precision.
    The double coefficients are the highest parts of the quad doubles.
    The first list in the tuple holds the coefficients of the numerator,
    the second list in the tuple holds the denominator coefficients.
    On entry in idx is the index of a variable.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_numerator_coefficient
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_denominator_coefficient
    numdeg = get_degree_of_numerator()
    dendeg = get_degree_of_denominator()
    numcfs = []
    for col in range(numdeg+1):
        (rcf, icf) = py2c_padcon_quaddobl_numerator_coefficient(idx, col, 0)
        numcfs.append(complex(rcf, icf))
    dencfs = []
    for col in range(dendeg+1):
        (rcf, icf) = py2c_padcon_quaddobl_denominator_coefficient(idx, col, 0)
        dencfs.append(complex(rcf, icf))
    return (numcfs, dencfs)

def standard_pade_vector(dim):
    """
    Returns the list of all coefficients over all dim variables,
    computed by the predictor in standard double precision.
    """
    result = []
    for i in range(1, dim+1):
        result.append(standard_pade_coefficients(i))
    return result

def dobldobl_pade_vector(dim):
    """
    Returns the list of all coefficients over all dim variables,
    computed by the predictor in double double precision.
    The double coefficients are the high parts of the double doubles.
    """
    result = []
    for i in range(1, dim+1):
        result.append(dobldobl_pade_coefficients(i))
    return result

def quaddobl_pade_vector(dim):
    """
    Returns the list of all coefficients over all dim variables,
    computed by the predictor in quad double precision.
    The double coefficients are the highest parts of the quad doubles.
    """
    result = []
    for i in range(1, dim+1):
        result.append(quaddobl_pade_coefficients(i))
    return result

def symbolic_pade_approximant(cff):
    """
    Given in cff are the coefficients of numerator and denominator
    of a Pade approximant, given as a tuple of two lists.
    On return is the string representation of the Pade approximant,
    using 't' as the variable.
    """
    (nq, dq) = cff
    num = ['+' + str(nq[k]) + ('*t**%d' % k) for k in range(len(nq))]
    numerator = ''.join(num)
    den = ['+' + str(nq[k]) + ('*t**%d' % k) for k in range(len(dq))]
    denominator = ''.join(den)
    result = '(' + numerator + ')/(' + denominator + ')'
    return result

def symbolic_pade_vector(cff):
    """
    Given in cff are the coefficients of numerator and denominator
    of a Pade vector, given as a list of tuples of two lists each.
    On return is the string representation of the Pade approximant,
    using 't' as the variable.
    """
    result = []
    for coefficients in cff:
        result.append(symbolic_pade_approximant(coefficients))
    return result

def standard_poles(dim):
    """
    Returns a list of lists of all poles of the vector of length dim,
    computed by the predictor in standard double precision.
    """
    from phcpy.phcpy2c2 import py2c_padcon_standard_pole
    dendeg = get_degree_of_denominator()
    result = []
    for i in range(1, dim+1):
        poles = []
        for j in range(1, dendeg+1):
            (repole, impole) = py2c_padcon_standard_pole(i, j, 0)
            poles.append(complex(repole, impole))
        result.append(poles)
    return result

def dobldobl_poles(dim):
    """
    Returns a list of lists of all poles of the vector of length dim,
    computed by the predictor in double double precision.
    The doubles in the poles are the high parts of the double doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_dobldobl_pole
    dendeg = get_degree_of_denominator()
    result = []
    for i in range(1, dim+1):
        poles = []
        for j in range(1, dendeg+1):
            (repole, impole) = py2c_padcon_dobldobl_pole(i, j, 0)
            poles.append(complex(repole, impole))
        result.append(poles)
    return result

def quaddobl_poles(dim):
    """
    Returns a list of lists of all poles of the vector of length dim,
    computed by the predictor in quad double precision.
    The doubles in the poles are the highest parts of the quad doubles.
    """
    from phcpy.phcpy2c2 import py2c_padcon_quaddobl_pole
    dendeg = get_degree_of_denominator()
    result = []
    for i in range(1, dim+1):
        poles = []
        for j in range(1, dendeg+1):
            (repole, impole) = py2c_padcon_quaddobl_pole(i, j, 0)
            poles.append(complex(repole, impole))
        result.append(poles)
    return result

def standard_next_track(target, start, sols, verbose=False, homogeneous=False):
    r"""
    Runs the series-Pade tracker step by step in double precision,
    for an artificial-parameter homotopy.
    On input are a target system and a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    If *homogeneous*, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(start)
    standard_set_homotopy(target, start, verbose, homogeneous)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        standard_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                standard_predict_correct(verbose)
                sol = standard_get_solution(verbose)
                print sol
                polestep = standard_pole_step()
                estidist = standard_estimated_distance()
                curvstep = standard_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (standard_t_value(), standard_step_size())
                frp = standard_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = standard_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', standard_series_coefficients(dim)
                print '3) Pade vector:', standard_pade_vector(dim)
                print '4) poles:', standard_poles(dim)
    return result

def dobldobl_next_track(target, start, sols, verbose=False, homogeneous=False):
    r"""
    Runs the series-Pade tracker step by step in double double precision,
    for an artificial-parameter homotopy.
    On input are a target system and a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    If *homogeneous*, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(start)
    dobldobl_set_homotopy(target, start, verbose, homogeneous)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        dobldobl_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                dobldobl_predict_correct(verbose)
                sol = dobldobl_get_solution(verbose)
                print sol
                polestep = dobldobl_pole_step()
                estidist = dobldobl_estimated_distance()
                curvstep = dobldobl_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (dobldobl_t_value(), dobldobl_step_size())
                frp = dobldobl_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = dobldobl_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', dobldobl_series_coefficients(dim)
                print '3) Pade vector:', dobldobl_pade_vector(dim)
                print '4) poles:', dobldobl_poles(dim)
    return result

def quaddobl_next_track(target, start, sols, verbose=False, homogeneous=False):
    r"""
    Runs the series-Pade tracker step by step in quad double precision,
    for an artificial-parameter homotopy.
    On input are a target system and a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    If *homogeneous*, then path tracking happens in projective space,
    otherwise the original affine coordinates are used.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(start)
    quaddobl_set_homotopy(target, start, verbose, homogeneous)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        quaddobl_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                quaddobl_predict_correct(verbose)
                sol = quaddobl_get_solution(verbose)
                print sol
                polestep = quaddobl_pole_step()
                estidist = quaddobl_estimated_distance()
                curvstep = quaddobl_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (quaddobl_t_value(), quaddobl_step_size())
                frp = quaddobl_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = quaddobl_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', quaddobl_series_coefficients(dim)
                print '3) Pade vector:', quaddobl_pade_vector(dim)
                print '4) poles:', quaddobl_poles(dim)
    return result

def standard_next_loop(hom, idx, sols, verbose=False):
    r"""
    Runs the series-Pade tracker step by step in double precision.
    On input is a natural parameter homotopy with solutions.
    The *hom* is a list of strings representing the polynomials
    of the natural parameter homotopy.
    The *idx* is the index of the variable in hom which is
    the continuation parameter.
    The *sols* is a list of strings representing start solutions.
    The start solutions do *not* contain the value of the continuation
    parameter, which is assumed to be equal to zero.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(hom) - 1
    standard_set_parameter_homotopy(hom, idx,verbose)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        print 'tracking solution path', idx, '...'
        standard_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                standard_predict_correct(verbose)
                sol = standard_get_solution(verbose)
                print sol
                polestep = standard_pole_step()
                estidist = standard_estimated_distance()
                curvstep = standard_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (standard_t_value(), standard_step_size())
                frp = standard_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = standard_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', standard_series_coefficients(dim)
                print '3) Pade vector:', standard_pade_vector(dim)
                print '4) poles:', standard_poles(dim)
    return result

def dobldobl_next_loop(hom, idx, sols, verbose=False):
    r"""
    Runs the series-Pade tracker step by step in double double precision.
    On input is a natural parameter homotopy with solutions.
    The *hom* is a list of strings representing the polynomials
    of the natural parameter homotopy.
    The *idx* is the index of the variable in hom which is
    the continuation parameter.
    The *sols* is a list of strings representing start solutions.
    The start solutions do *not* contain the value of the continuation
    parameter, which is assumed to be equal to zero.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(hom) - 1
    dobldobl_set_parameter_homotopy(hom, idx,verbose)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        print 'tracking solution path', idx, '...'
        dobldobl_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                dobldobl_predict_correct(verbose)
                sol = dobldobl_get_solution(verbose)
                print sol
                polestep = dobldobl_pole_step()
                estidist = dobldobl_estimated_distance()
                curvstep = dobldobl_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (dobldobl_t_value(), dobldobl_step_size())
                frp = dobldobl_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = dobldobl_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', dobldobl_series_coefficients(dim)
                print '3) Pade vector:', dobldobl_pade_vector(dim)
                print '4) poles:', dobldobl_poles(dim)
    return result

def quaddobl_next_loop(hom, idx, sols, verbose=False):
    r"""
    Runs the series-Pade tracker step by step in quad double precision.
    On input is a natural parameter homotopy with solutions.
    The *hom* is a list of strings representing the polynomials
    of the natural parameter homotopy.
    The *idx* is the index of the variable in hom which is
    the continuation parameter.
    The *sols* is a list of strings representing start solutions.
    The start solutions do *not* contain the value of the continuation
    parameter, which is assumed to be equal to zero.
    The function is interactive, prompting the user each time
    before performing the next predictor-corrector step.
    If *verbose*, then extra output is written.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.solver import number_of_symbols
    result = []
    dim = number_of_symbols(hom) - 1
    quaddobl_set_parameter_homotopy(hom, idx,verbose)
    (idx, tval) = (0, 0.0)
    fmt = 'pole step : %.3e, estimated distance : %.3e, Hessian step : %.3e'
    for sol in sols:
        idx = idx + 1
        print 'tracking solution path', idx, '...'
        quaddobl_set_solution(dim, sol, verbose)
        while(True):
            answer = raw_input('next predictor-corrector step ? (y/n) ')
            if(answer != 'y'):
                result.append(sol)
                break
            else:
                quaddobl_predict_correct(verbose)
                sol = quaddobl_get_solution(verbose)
                print sol
                polestep = quaddobl_pole_step()
                estidist = quaddobl_estimated_distance()
                curvstep = quaddobl_hessian_step()
                print fmt % (polestep, estidist, curvstep)
                previoustval = tval
                (tval, step) = (quaddobl_t_value(), quaddobl_step_size())
                frp = quaddobl_pole_radius()
                print "t : %.3e, step : %.3e, frp : %.3e" % (tval, step, frp)
                cfp = quaddobl_closest_pole()
                print 'For the previous t value', previoustval, ':'
                print '1) closest pole : ', cfp
                print '2) the series:', quaddobl_series_coefficients(dim)
                print '3) Pade vector:', quaddobl_pade_vector(dim)
                print '4) poles:', quaddobl_poles(dim)
    return result

def test_hyperbola(precision='d'):
    """
    Tests the step-by-step Pade tracker on a hyperbola,
    by default in double precision (when precision equals 'd').
    Other supported precisions are double double ('dd')
    and quad double ('qd').
    """
    from math import sqrt
    from phcpy.curves import tune_homotopy_continuation_parameters
    from phcpy.solutions import make_solution
    tune_homotopy_continuation_parameters()
    par = 0.1
    xtp = ['x^2 - (t - 0.5)^2 - 0.01;']
    # print 'initializing the homotopy :', xtp[0]
    # standard_set_parameter_homotopy(hom=xtp, idx=2, verbose=True)
    print 
    print 'value of the first start solution :',
    solx = sqrt(4*par**2+1)/2
    print solx
    sol1 = make_solution(['x'], [solx])
    print 'the first start solution :\n', sol1
    sol2 = make_solution(['x'], [-solx])
    print 'the second start solution :\n', sol2
    # standard_set_solution(nvar=1, sol=sol1, verbose=True)
    # standard_predict_correct(verbose=True)
    ans = raw_input('Verbose ? (y/n) ')
    vrb = (ans == 'y')
    if precision == 'd':
        print 'tracking in double precision ...'
        standard_next_loop(xtp, 2, [sol1, sol2], vrb)
    elif precision == 'dd':
        print 'tracking in double double precision ...'
        dobldobl_next_loop(xtp, 2, [sol1, sol2], vrb)
    elif precision == 'qd':
        print 'tracking in quad double precision ...'
        quaddobl_next_loop(xtp, 2, [sol1, sol2], vrb)
    else:
        print 'wrong value for the precision'

def test_simple_track(precision='d'):
    """
    Tunes the parameters and runs a simple test on the trackers.
    The precision is either double ('d'), double double ('dd'),
    or quad double ('qd').
    """
    pars = [get_homotopy_continuation_parameter(k) for k in range(1, 13)]
    print pars
    print 'gamma constant :', get_gamma_constant()
    print 'degree of numerator :', get_degree_of_numerator()
    print 'degree of denominator :', get_degree_of_denominator()
    print 'maximum step size :', get_maximum_step_size()
    print 'minimum step size :', get_minimum_step_size()
    print 'series beta factor :', get_series_beta_factor()
    print 'pole radius beta factor :', get_pole_radius_beta_factor()
    print 'predictor residual alpha :', get_predictor_residual_alpha()
    print 'corrector residual tolerance :', get_corrector_residual_tolerance()
    print 'coefficient tolerance :',  get_zero_series_coefficient_tolerance()
    print 'maximum #corrector steps :', get_maximum_corrector_steps()
    print 'maximum #steps on path :',  get_maximum_steps_on_path()
    tune_homotopy_continuation_parameters()
    from phcpy.families import katsura
    k3 = katsura(3)
    from phcpy.solver import total_degree_start_system as tdss
    (k3q, k3qsols) = tdss(k3)
    print 'tracking', len(k3qsols), 'paths ...'
    if(precision == 'd'):
        k3sols = standard_track(k3, k3q, k3qsols, "/tmp/out", True, 1)
    elif(precision == 'dd'):
        k3sols = dobldobl_track(k3, k3q, k3qsols, "/tmp/out", True, 1)
    elif(precision == 'qd'):
        k3sols = quaddobl_track(k3, k3q, k3qsols, "/tmp/out", True, 1)
    else:
        print 'wrong precision'
    for sol in k3sols:
        print sol

def test_next_track(precision='d'):
    """
    Tunes the parameters and runs the step-by-step tracker.
    The precision is either double ('d'), double double ('dd'),
    or quad double ('qd').
    """
    tune_homotopy_continuation_parameters()
    from phcpy.families import katsura
    k3 = katsura(3)
    from phcpy.solver import total_degree_start_system as tdss
    (k3q, k3qsols) = tdss(k3)
    ans = raw_input('homogeneous coordinates ? (y/n) ')
    homo = (ans == 'y')
    print 'tracking', len(k3qsols), 'paths ...'
    if(precision == 'd'):
        k3sols = standard_next_track(k3, k3q, k3qsols, True, homo)
    elif(precision == 'dd'):
        k3sols = dobldobl_next_track(k3, k3q, k3qsols, True, homo)
    elif(precision == 'qd'):
        k3sols = quaddobl_next_track(k3, k3q, k3qsols, True, homo)
    else:
        print 'wrong precision'
    for sol in k3sols:
        print sol

def test_twohom(precision='d'):
    """
    Test on a 2-homogeneous homotopy.
    """
    from phcpy.solver import linear_product_root_count as lprc
    from phcpy.solver import random_linear_product_system as rlps
    tws = ['x^2 + x - 1;', 'x*y + x - y + 3;']
    roco = lprc(tws)
    twsq, twsqsols = rlps(tws)
    print 'the start system :'
    for pol in twsq: print pol
    print 'the start solutions :'
    for sol in twsqsols: print sol
    if(precision == 'd'):
        sols = standard_track(tws, twsq, twsqsols, "/tmp/out", True, 2, [1, 2])
    elif(precision == 'dd'):
        sols = dobldobl_track(tws, twsq, twsqsols, "/tmp/out", True, 2, [1, 2])
    elif(precision == 'qd'):
        sols = quaddobl_track(tws, twsq, twsqsols, "/tmp/out", True, 2, [1, 2])
    else:
        print 'wrong value for the precision'
    for sol in sols:
        print sol

if __name__ == "__main__":
    # test_next_track()
    # test_simple_track('dd')
    # test_simple_track('qd')
    test_hyperbola()



#!/usr/bin/python

# Enter debug mode
from phc_config import phc_debug, style_reg
if phc_debug:
    import cgitb
    cgitb.enable()
   
import cgi, Cookie, os, sha, datetime, time

from phc_html import PrintHeader, Footer_Bar
from phc_sql import sql_reg

"""
infos[0]: Email
infos[1]: password
infos[2]: repeated password
infos[3]: Name_First
infos[4]: Name_Last
infos[5]: Org

error[0]: 1. general error
          2. password don't match
error[1]: infos[0]: Email
error[2]: infos[1]: password
error[3]: infos[2]: repeated password
error[4]: infos[3]: Name_First
error[5]: infos[4]: Name_Last
error[6]: infos[5]: Org
"""

def ProcessName(form):
   """
   Processes name of Email form.
   Returns True if error, else False.
   """
   error = [0,0,0,0,0,0,0]
   infos = ["","","","","",""]
   try:
      infos[0] = form['login'].value
   except KeyError:
      #print "Please enter your email"
      error[1] = 1
      error[0] = 1

   # password
   try:
      infos[1] = form['passw'].value
   except KeyError:
      #print "Please enter your password"
      error[2] = 1
      error[0] = 1
   try:
      infos[2] = form['passw_check'].value
   except KeyError:
      #print "Please enter your password"
      error[3] = 1
      error[0] = 1
   try:
      infos[3] = form['Name_First'].value
   except KeyError:
      error[4] = 1
      error[0] = 1
   try:
      infos[4] = form['Name_Last'].value
   except KeyError:
      error[5] = 1
      error[0] = 1
   try:
      infos[5] = form['Org'].value
   except KeyError:
      error[6] = 1
      error[0] = 1
   # check if password match
   if error[0] == 0:
      if infos[1] != infos[2]:
         error[0] = 2
   return infos, error

def handle_error(error):
   if error[0] == 1:
      print "Please enter these information"
      info = ["Email", "Password","Password Confirm",  "First Name", "Last Name", "Organization" ]
      for i in xrange(0, len(info)):
         if error[i+1]==1:
            print ", " + info[i]
   elif error[0] == 2:
      print "Sorry. Your password don't match."
   elif error[0] == 3:
      print """Sorry. Your email has been registered. 
         <a href='Find_Password.html'>Find my password</a>"""
   elif error[0] == 4:
      print """Sorry. We couldn't send the activation email to your mailbox. Please check your email address"""

def main():
   """
   Form to process Email.
   """
   PrintHeader(style_reg)
   form = cgi.FieldStorage()
   infos, error = ProcessName(form)
   if not error[0]:
      error[0], ticket = sql_reg(infos)
      if not error[0]:
         print """
               Congratulations! Register successful.</p>
               Please check your mailbox to activate your account."""
         
   if error[0]:
      print """<html><body>
               <div>
               <h1> Register </h1>
               <form method = "post" autocomplete="off"
                     action = "register.py">"""
      print """<table border="0">"""


      form_list = [[infos[0], 'Email'           , 'login'      ],\
                   [infos[1], 'Password'        , 'passw'      ],\
                   [infos[2], 'Confirm Password', 'passw_check'],\
                   [infos[3], 'First Name'      , 'Name_First' ],\
                   [infos[4], 'Last Name'       , 'Name_Last'  ],\
                   [infos[5], 'Organization'    , 'Org'        ]]
      
      # First Name
      for i in xrange(len(form_list)):
         print """<tr><td class="reg_form">%s</td>"""%form_list[i][1]
         if i==1 or i==2:
            print """<td><input type="password" name="%s" maxlength="15">
                     </td></tr>"""%form_list[i][2]
         else:
            if form_list[i][0] == '':
               print """<td><input type="text" name="%s" spellcheck="false" >
                        </td></tr>"""%form_list[i][2]
            else:
               print """<td><input type = "text" name = "%s"  
                      spellcheck="false"  value =%s></td><td>V</td>"""\
                     %(form_list[i][2],form_list[i][0])
            print "</tr>"

      print """</table>
         <input type="submit" value="Submit">
         </form>
         </div>"""
   handle_error(error)
   Footer_Bar()
   print "</body></html>\n"

main()




from phc_html import *
from phc_file import get_file_id, check_file_exist
from os import path
from phc_client import PHC_Server

def PHC_Edit_Hom(Uid,form):  
   poly_id = form['poly_id'].value
   file_id = get_file_id(Uid, poly_id)
   if not path.isfile(file_id+".start"):
      data = PHC_Server(Uid, '7', poly_id)
      if data != 0 and path.isfile(file_id+".start"):
         Print_Poly_File_New(Uid, poly_id, 1)
      else:
         Print_Poly_File_New(Uid, poly_id, 0)
         print """<p><font color='red'><b>Error:</b> the original system doesn't have a start system. Please just solve it directly.</font></p>"""
         
   else:
      Print_Poly_File_New(Uid, poly_id, 1)

def PHC_Edit(Uid, form):
   if form.has_key('homotopy'):
      PHC_Edit_Hom(Uid,form)

   elif form.has_key('poly'):
      poly = form['poly'].value
      Print_Poly_New(Uid, poly)

   elif form.has_key('poly_id'):
      poly_id = form['poly_id'].value
      Print_Poly_File_New(Uid, poly_id)

   else:
      Print_Poly_Empty(Uid)



"""
Making a witness set of the twisted cubic.
The twisted cubic is a curve in 3-space of degree 3,
defined as the intersection of a quadratic and a cubic cylinder.
Adding a random plane to the original system leads to three
generic points on the cubic.  These three points, jointly
with the embedded system form a witness set for the twisted cubic.
"""
twisted = ['x^2 - y;', 'x^3 - z;']
print 'polynomials that define the twisted cubic :'
for pol in twisted:
    print pol
from phcpy.sets import embed, cascade_step
from phcpy.solver import solve
et = embed(3, 1, twisted)
print 'polynomials in the embedded system :'
for pol in et:
    print pol
etsols = solve(et)
print 'the witness points :'
for point in etsols:
    print point



#!/usr/bin/python

# Enter debug mode
from phc_config import phc_debug
if phc_debug:
    import cgitb
    cgitb.enable()

import cgi, Cookie, os, sha, datetime

from sql_login import PHC_login
from phc_client import PHC_Server
from phc_html import *

from phc_config import style_web

from phc_login_err import *
from phc_solve import *
from phc_edit import *
from phc_update import *
from phc_preview import *
from phc_kill import *
from phc_mypoly import *

#####################     File Management Section    ######################

def PHC_Rename(Uid, s,t):
   return PHC_Server(Uid,'4',s+'|'+t)

def PHC_Delete(Uid,name):  
   return PHC_Server(Uid,'5',name)

def polypid(Uid):
   f = "poly.pid"
   file = open(f,'r')
   Pid = file.readline()
   file.close()
   return '1'

############################  PHC Function Section ############################

def PHCWEB(form,Uid, Status, phcdb): ### combine all these key to phcaction
   from os import path

   # Job 0 and 1: Solve polynomial by homotopy
   if form.has_key('Solve'):
      if phc_debug: print "<p>Action: Solve</p>"
      PHC_Solve(Uid, Status, form, phcdb)

   # Job 2: Kill
   elif form.has_key('Kill'):
      if phc_debug: print "<p>Action: Kill</p>"
      PHC_Kill(Uid, form, phcdb)

   # Manage 0: New Polynomial
   elif form.has_key('New'):
      if phc_debug: print "<p>Action: New</p>"
      Print_Poly_Empty(Uid)

   # Manage 1: View Polynomials
   elif form.has_key('print'):
      if phc_debug: print "<p>Action: Print</p>"
      PHC_Print(Uid,form)

   # Manage 2: Save Polynomials XXX rename need to be done
   elif form.has_key('Save'):
      if phc_debug: print "<p>Action: Save</p>"
      poly_id = form['poly_id'].value
      new_poly_id = form['newname'].value
      data = PHC_Rename(Uid, poly_id, new_poly_id)
      if data != 0:
         form['poly_id'].value = new_poly_id
         PHC_Update(Uid, '0', new_poly_id, phcdb)
      else:
         PHC_Update(Uid, '0', poly_id, phcdb)

   # Manage 3: Delete Polynomials
   elif form.has_key('Delete'):
      if phc_debug: print "<p>Action: Delete</p>"
      if form.has_key('poly_id'):
         poly_id = form['poly_id'].value
      else:
         poly_id = ''
      poly_id_del = form['Delete'].value
      data = PHC_Delete(Uid,poly_id_del)
      PHC_Update(Uid, '0', poly_id, phcdb)

   # Manage 6: Preview
   elif form.has_key('Preview'):
      if phc_debug: print "<p>Action: Preview</p>"
      PHC_Preview(Uid, form)

   # Manage 4: Edit
   elif form.has_key('Edit') or form.has_key('homotopy'):
      if phc_debug: print "<p>Action: Edit\n"
      PHC_Edit(Uid, form)

   # Manage 5: Update
   elif form.has_key('Update') :
      if phc_debug: print "<p>Action: Update</p>"
      Pid = form['Pid'].value
      PHC_Update(Uid,Pid,"current",phcdb)

   # Manage 7: view poly
   elif form.has_key('poly_id'):
      if phc_debug: print "<p>Action: Update2</p>"
      poly_id = form['poly_id'].value
      sol_page = 0
      if form.has_key('sol_page'):
         sol_page = int(form['sol_page'].value)
      PHC_Update(Uid,'0',poly_id,phcdb, sol_page)

   # Manage 8: User Come back
   elif Status != None:
      if phc_debug: print "<p>Action: Update</p>"
      PHC_Update(Uid,'0',"current",phcdb)

   # Manage 0: New Polynomial
   else:
      if phc_debug: print "<p>Action: New</p>"
      Print_Poly_Empty(Uid)
   
   PHC_Mypoly(form, Uid, phcdb)


########################   Main Function    #############################

def main():
   """
   Form to process Email.
   """
   form = cgi.FieldStorage()
   # Processing cookie and user pass ### put this part into cookie_login.py
   from phc_sql import PHCSQL
   phcdb = PHCSQL()
   error, Uid, Name_First, Status = PHC_login(form,phcdb)
   # PHCpack Web Interface Header
   PrintHeader(style_web)
   #print "Uid = %s</p>" % Uid
   if not error or error == 7: # error 7: folder not exist, recovery
      Print_User_Bar(Name_First)
      PHCWEB(form,Uid, Status, phcdb)
      # Need to close database
   else:
      handle_error(error)
   phcdb.db.close()
   Footer_Bar()
   print "</div></body></html>\n"

main()




"""
Python3 script to compute with sympy the sines and cosines
of multiples of pi/1024 with 256 decimal places.
The output is formatted as Ada assignments of constant strings.
"""
from sympy import pi, sin, cos, evalf
S = []
C = []
for k in range(1, 257):
    p = k*pi/1024
    s = sin(p).evalf(256)
    S.append(s)
    c = cos(p).evalf(256)
    C.append(c)
print('sine table :')
for k in range(len(S)):
    print('    sint%03d : constant string' % k)
    s = str(S[k])
    L = len(s)//5
    (s0, s1, s2, s3, s4) = (s[0:L], s[L:2*L], s[2*L:3*L], s[3*L:4*L], s[4*L:])
    print('            := \"%s\"' % s0)
    print('             & \"%s\"' % s1)
    print('             & \"%s\"' % s2)
    print('             & \"%s\"' % s3)
    print('             & \"%s\";' % s4)
print('cosine table :')
for k in range(len(C)):
    print('    cost%03d : constant string' % k)
    c = str(C[k])
    L = len(s)//5
    (c0, c1, c2, c3, c4) = (c[0:L], c[L:2*L], c[2*L:3*L], c[3*L:4*L], c[4*L:])
    print('            := \"%s\"' % c0)
    print('             & \"%s\"' % c1)
    print('             & \"%s\"' % c2)
    print('             & \"%s\"' % c3)
    print('             & \"%s\";' % c4)



"""
Script to generate polynomial systems in the homotopy to solve
hypersurface Pieri problems in the numerical Schubert calculus.
The script needs sympy for the symbolic representation of the variable
entries in a matrix.  To compute determinants, numpy.linalg.det is used.
"""

import sympy as sp
from numpy import ndarray, zeros
from numpy.linalg import det, qr
from random import uniform
from math import sin, cos, pi

def random_matrix(rows, cols):
    """
    Returns a numpy array of shape rows and cols
    to store the data for a random complex matrix
    of as many rows as the value of rows and
    as many columns as the value of cols.
    The entries are random complex numbers
    of modulus one.  For numerical conditioning,
    the columns of the random matrix are orthonormalized.
    """
    numbers = ndarray(shape=(rows, cols), dtype=complex)
    for i in range(rows):
        for j in range(cols):
            angle = uniform(0, 2*pi)
            numbers[i, j] = complex(cos(angle), sin(angle))
    result, upper = qr(numbers)
    return result

def double2linear(nbrows, nbcols, row, col):
    """
    Returns one single index to locate the variable
    at position (row, col) in a variable matrix with
    as many rows as the value of nbrows and
    as many columns as the value of nbcols.
    The counting of indices starts at one
    and the matrix has a banded shape, with the ones
    in the identity matrix starting on top.
    The first variables start at the last column.
    """
    nvc = nbrows - nbcols    # number of variables per column
    prv = (nbcols - col)*nvc # number of preceding variables
    idx = row - col          # row index w.r.t. identity
    return prv + idx

def variables(rows, cols, pivots, linear=False):
    """
    Returns a symbolic matrix with as many rows
    as the value of rows and as many columns as
    the value of cols.  The pivots determine the
    position of the last nonzero row.
    The length of the list pivots must equal the
    value of the parameter cols.
    If linear, then the names of the variables
    will following a linear indexing scheme as
    defined by the function double2linear.
    """
    result = sp.Matrix(rows, cols, lambda i, j: 0)
    for i in range(cols):
        result[i, i] = 1
    for k in range(cols):
        for i in range(k+1, pivots[k]+1):
            if linear:
                idx = double2linear(rows, cols, i+1, k+1)
                name = 'x' + str(idx)
            else:
                name = 'x' + '_' + str(i+1) + '_' + str(k+1)
            sp.var(name)
            result[i, k] = eval(name)
    return result

def enumerate_all_minors(mat, rows, cols, crnt, accu):
    """
    Prints the row indices that define all minors
    of the matrix mat of dimension equal to cols.
    The matrix must have as many rows as the value of rows
    and as many columns as the value of cols.
    The variable crnt is the current column and accu
    accumulates the current selection of rows.
    """
    if(crnt >= cols):
        print accu
    else:
        start = -1
        if(len(accu) > 0):
            start = accu[-1]
        for i in range(start+1, rows):
            if(mat[i, crnt] != 0):
                enumerate_all_minors(mat, rows, cols, crnt+1, accu+[i])

def complement(cff, rows, cols, idx):
    """
    Returns the determinant of the complement of the selected
    rows as defined by the list idx of the matrix cff.
    """
    dim = rows - cols
    mat = ndarray(shape=(dim, dim), dtype=complex)
    cnt = -1
    for row in range(rows):
        if(not (row in idx)):
            cnt = cnt + 1
            for col in range(dim):
                mat[cnt, col] = cff[row, col]
    return det(mat)

def make_term(mat, rows, cols, idx, cffs):
    """
    Returns the string representation of the term defined by
    the selection of rows in the list idx.
    If the coefficient is zero, then the empty string is returned.
    """
    coeff = complement(cffs, rows, cols, idx)
    if(coeff == 0.0):
        result = ''
    else:
        cffre = '%.16e' % coeff.real
        cffim = '%+.16e' % coeff.imag
        result = '(' + cffre + cffim + '*i)'
        for k in range(cols):
            if(mat[idx[k], k] != 1):
                if(result == ''):
                    result = str(mat[idx[k],k])
                else:
                    result = result + '*' + str(mat[idx[k],k])
    return result

def enumerate_all_terms(mat, rows, cols, crnt, accu, cffs, equ):
    """
    Computes all terms in the Pieri condition imposed on the matrix mat
    with coefficients in the matrix cffs.
    The matrix mat has as many rows as the value of rows
    and as many columns as the value of cols.
    The matrix cffs has as many rows as the value of rows
    and as many columns as the value of rows - cols.
    The function assumes the matrix to have a banded shape.
    The variable crnt is the current column and accu
    accumulates the current selection of rows.
    The terms are accumulated as strings in the list equ.
    """
    if(crnt >= cols):
        term = make_term(mat, rows, cols, accu, cffs)
        equ.append(term)
    else:
        start = -1
        if(len(accu) > 0):
            start = accu[-1]
        for i in range(start+1, rows):
            if(mat[i, crnt] != 0):
                enumerate_all_terms(mat, rows, cols, crnt+1, accu+[i], \
                                    cffs, equ)

def all_minors(mat, rows, cols):
    """
    Prints the row indices that define all minors
    of the matrix mat of dimension equal to cols.
    The matrix must have as many rows as the value of rows
    and as many columns as the value of cols.
    """
    enumerate_all_minors(mat, rows, cols, 0, [])

def all_terms(mat, rows, cols, cffs):
    """
    Returns all terms in the Pieri condition imposed on the variable
    matrix mat by the coefficients in the matrix cffs.
    The matrix must have as many rows as the value of rows
    and as many columns as the value of cols.
    The numerical matrix cffs has as many rows as the value of rows
    and as many columns as the value of rows - cols.
    """
    result = []
    enumerate_all_terms(mat, rows, cols, 0, [], cffs, result) 
    return result

def sum_terms(equ):
    """
    Given in equ a list of string representing terms,
    returns the sum of the terms as a string.
    """
    result = ''
    if(len(equ) > 0):
        if(equ[0] != ''):
            result = equ[0]
        for i in range(1, len(equ)):
            if(equ[i] != ''):
                result = result + ' + ' + equ[i]
    return result

def number_of_variables(pivots):
    """
    Returns the number of variables in the variable matrix
    defined by the pivots.
    """
    result = 0
    for k in range(len(pivots)):
        result = result + pivots[k] - k;
    return result

def ask_inputs():
    """
    Prompts the user for the number of rows, 
    the number of columns, and pivots.
    These inputs are returned as a tuple.
    """
    rawnbr = raw_input("Give the number of rows : ")
    rawnbc = raw_input("Give the number of columns : ")
    rows = int(rawnbr)
    cols = int(rawnbc)
    pvts = [k for k in range(cols)]
    # pvts[cols-1] = rows-1 # rows - 1 is largest linear system
    pvts[cols-1] = rows-2   # rows - 2 allows for special plane
    print '-> the start pivots :', pvts
    print '-> number of variables :', number_of_variables(pvts)
    answer = raw_input('Do you want to give other pivots ? (y/n) ')
    if(answer == 'y'):
        rawpvts = raw_input('Give a list of increasing numbers : ')
        pvts = eval(rawpvts)
        print '-> your pivots :', pvts
        print '-> number of variables :', number_of_variables(pvts)
    return rows, cols, pvts

def show_variable_matrix():
    """
    Prompts the user for the number of rows and columns,
    the pivot information of the matrix of variables.
    Then shows the variable matrix and all its minors,
    along with the equation for a random coefficient matrix.
    """
    rows, cols, pvts = ask_inputs()
    answer = raw_input('Double indexing of variables ? (y/n) ')
    linear = (answer != 'y')
    varmat = variables(rows, cols, pvts, linear)
    print '-> the matrix of variables :'
    print varmat
    answer = raw_input('Enumerate all minors ? (y/n) ')
    if(answer == 'y'):
        all_minors(varmat, rows, cols)
    answer = raw_input('Enumerate all terms ? (y/n) ')
    dim = rows - cols
    cff = random_matrix(rows, dim)
    equ = all_terms(varmat, rows, cols, cff)
    print '-> all terms in the Pieri condition :'
    print sum_terms(equ)

def rightmost_nonfull_pivot(dim, pivots):
    """
    Let dim be the dimension of the ambient space.
    The list of pivots is full if equal to 
    [dim - L, dim - L + 1, .., dim - 1], L = len(pivots).
    If the list of pivots is full, then -1 is returned,
    otherwise return the index of the rightmost entry
    in pivots that can still be increased.
    """
    L = len(pivots) 
    for k in range(L-1,-1,-1):
        if(pivots[k] < dim - L + k):
            return k
    return -1

def update_pivots(dim, pivots):
    """
    Returns the updated pivots after incrementing
    the rightmost nonfull pivot in pivots.
    If the pivots are full, then pivots are returned.
    """
    piv = rightmost_nonfull_pivot(dim, pivots)
    if(piv < 0):
        result = pivots
    else:
        result = [x for x in pivots]
        result[piv] = result[piv] + 1
    return result

def extend_solutions(sols, dim, pivots):
    """
    Adds one extra zero coordinate to all solutions in sols,
    corresponding to the rightmost nonfull pivot.
    """
    from phcpy.solutions import strsol2dict, variables, make_solution
    piv = rightmost_nonfull_pivot(dim, pivots)
    print '-> in extend_solutions, piv =', piv
    if(piv < 0):
        result = sols
    else:
        rown = pivots[piv] + 1  # row number of new variable 
        name = 'x_' + str(rown+1) + '_' + str(piv+1)
        print '-> the name of the new variable :', name
        result = []
        for sol in sols:
            dicsol = strsol2dict(sol)
            solvar = variables(dicsol)
            solval = [dicsol[var] for var in solvar] 
            solvar.append(name)
            solval.append(complex(0.0))
            result.append(make_solution(solvar, solval))
    return result

def special_plane(dim, pivots):
    """
    Returns a matrix with as many rows as the value of dim
    and as many columns as the value of dim - len(pivots).
    The plane is a special plane so the Pieri condition is
    satisfied when the variable with the rightmost nonfull
    pivot is set to zero.
    """
    cols = dim - len(pivots)
    result = zeros(shape=(dim, cols), dtype=complex)
    ind = 0
    for k in range(dim):
        if(not (k in pivots)):
            (result[k, ind], ind) = (1.0, ind+1)
    return result

def pieri_system(rows, cols, pivots, planes, linear=False):
    """
    Generates a Pieri system for the numbers of rows in rows,
    the number of columns in cols, and the given pivots.
    The list planes must contain at least as many matrices
    as the number of variables in the pivots.
    The number of rows of the matrices in planes is the value
    of rows and the number of columns of the matrices in planes
    must equal to rows - cols.
    If linear, then variables in the matrix are named with 
    a strict linear index instead of a double index.
    """
    mat = variables(rows, cols, pivots, linear)
    nvr = number_of_variables(pivots)
    dim = rows - cols
    result = []
    for k in range(nvr):
        cff = planes[k]
        equ = all_terms(mat, rows, cols, cff)
        print '-> number of terms in equation %d : %d' % (k+1, len(equ))
        pol = sum_terms(equ)
        result.append(pol + ';')
    return result

def random_planes(rows, cols, pivots):
    """
    Generates as many random planes as the number of variables
    described by the list of pivots.
    The dimension of the planes is the value of cols
    in a space of dimension equal to the value of rows.
    """
    nvr = number_of_variables(pivots)
    dim = rows - cols
    planes = []
    for k in range(nvr):
        cff = random_matrix(rows, dim)
        planes.append(cff)
    return planes

def random_pieri_system(rows, cols, pivots, linear=False):
    """
    Generates a Pieri system for the numbers of rows in rows,
    the number of columns in cols, and the given pivots.
    The conditions are formulated for random planes.
    If linear, then variables in the matrix are named with 
    a strict linear index instead of a double index.
    """
    planes = random_planes(rows, cols, pivots)
    return pieri_system(rows, cols, pivots, planes, linear)

def start_pieri_system(rows, cols, pivots, planes, linear=False):
    """
    Generates a Pieri system where the last equation
    is for a special plane.  For all other equations,
    the planes in the list planes will be used.
    If linear, then variables in the matrix are named with 
    a strict linear index instead of a double index.
    """
    newpiv = update_pivots(rows, pivots)
    mat = variables(rows, cols, newpiv, linear)
    nvr = number_of_variables(pivots)
    dim = rows - cols
    result = []
    for k in range(nvr):
        cff = planes[k]
        equ = all_terms(mat, rows, cols, cff)
        print '-> number of terms in equation %d : %d' % (k+1, len(equ))
        pol = sum_terms(equ)
        result.append(pol + ';')
    lastcff = special_plane(rows, newpiv)
    print '-> the special plane :'
    print lastcff
    lastequ = all_terms(mat, rows, cols, lastcff)
    lastpol = sum_terms(lastequ)
    result.append(lastpol + ';')
    return result

def random_start_pieri_system(rows, cols, pivots, linear=False):
    """
    Generates a Pieri system where the last equation
    is for a special plane.  For all other equations,
    random matrices will be generated.
    If linear, then variables in the matrix are named with 
    a strict linear index instead of a double index.
    """
    planes = random_planes(rows, cols, pivots)
    return start_pieri_system(rows, cols, pivots, planes, linear) 

def solve_pieri_system(eqs):
    """
    Applies the black box solver of phcpy to solve the equations
    defined by the strings in the list eqs.  Note that, because
    of its particular structure, this will only work well if the 
    system is linear.
    """
    from phcpy.solver import solve
    sols = solve(eqs)
    print '-> the solutions :'
    for sol in sols:
        print sol
    return sols

def write_to_name_file(name, eqs, sols, dimspace, dimplane):
    """
    Opens a file with the given name for writing and the writes the
    system defined by the string representations of polynomials to file.
    If the list sols is not empty, then the solutions will be
    writen as well to file.  The parameters dimspace and dimplane
    respectively represent the dimensions of the space and 
    the solution planes.
    """
    file = open(name, 'w')
    file.write(str(len(eqs)) + '\n')
    for pol in eqs:
        file.write(pol + '\n')
    file.write('\n')
    file.write('TITLE : Pieri conditions on %d-planes in %d-space\n' \
               % (dimplane, dimspace))
    file.write('\n')
    if(len(sols) > 0):
        cnt = 0
        file.write('THE SOLUTIONS :\n')
        file.write(str(len(sols)) + ' ' + str(len(eqs)) + '\n')
        file.write('=====================================================\n')
        for sol in sols:
            cnt = cnt + 1
            file.write('solution ' + str(cnt) + ' : \n')
            file.write(sol + '\n')

def write_to_file(eqs, sols, dimspace, dimplane):
    """
    Prompts the user for a file name and then writes the system
    defined by the string representations of polynomials to file.
    If the list sols is not empty, then the solutions will be
    writen as well to file.  The parameters dimspace and dimplane
    respectively represent the dimensions of the space and 
    the solution planes.
    """
    name = raw_input('Give a name of a file to write to : ')
    write_to_name_file(name, eqs, sols, dimspace, dimplane)

def show_pieri_system():
    """
    Prompts the user for the number of rows and columns,
    the pivot information of the matrix of variables.
    Then shows the Pieri system for as many random matrices
    as there are variables in the matrix.
    """
    rows, cols, pvts = ask_inputs()
    answer = raw_input('Double indexing of variables ? (y/n) ')
    linear = (answer != 'y')
    answer = raw_input('Make start Pieri system ? (y/n) ')
    if(answer == 'y'):
        eqs = random_start_pieri_system(rows, cols, pvts, linear)
    else:
        eqs = random_pieri_system(rows, cols, pvts, linear)
    answer = raw_input('Do you want to see the polynomials ? (y/n) ')
    if(answer == 'y'):
        print '-> the polynomials in the Pieri system :'
        for pol in eqs:
            print pol
    answer = raw_input('Apply solve of phcpy ? (y/n) ')
    if(answer == 'y'):
        sols = solve_pieri_system(eqs)
    else:
        sols = []
    answer = raw_input('Write the system to file ? (y/n) ')
    if(answer == 'y'):
        write_to_file(eqs, sols, rows, cols)

def track_sequence_of_pieri_systems():
    """
    Makes a sequence of Pieri systems
    and runs the path trackers on one path.
    """
    from phcpy.trackers import track
    nbrows, nbcols, pvts = ask_inputs()
    answer = raw_input('Double indexing of variables ? (y/n) ')
    linear = (answer != 'y')
    dim = nbrows - nbcols
    planes = random_planes(nbrows, nbcols, pvts)
    start = start_pieri_system(nbrows, nbcols, pvts, planes, linear)
    startsols = solve_pieri_system(start)
    newpvts = update_pivots(nbrows, pvts)
    planes.append(random_matrix(nbrows, dim))
    target = pieri_system(nbrows, nbcols, newpvts, planes, linear)
    sols = track(target, start, startsols)
    print '-> the solutions after track :'
    for sol in sols:
        print sol
    while True:
        answer = raw_input('Continue to next level ? (y/n) ')
        if(answer != 'y'):
            break
        else:
            pvts = [piv for piv in newpvts]
            newpvts = update_pivots(nbrows, pvts)
            if(pvts == newpvts):
                print '-> no extension of pivots possible: no next level'
                break
            start = start_pieri_system(nbrows, nbcols, pvts, planes, linear)
            print '-> the new start system :'
            for pol in start:
                print pol
            startsols = extend_solutions(sols, nbrows, pvts)
            print '-> the extended solutions :'
            for sol in startsols:
                print sol
            planes.append(random_matrix(nbrows, dim))
            target = pieri_system(nbrows, nbcols, newpvts, planes, linear)
            print '-> the new target system :'
            for pol in target:
                print pol
            sols = track(target, start, startsols)
            print '-> the solutions after track :'
            for sol in sols:
                print sol
    answer = raw_input('Write system and solutions to file ? (y/n) ')
    if(answer == 'y'):
        write_to_file(target, sols, nbrows, nbcols)

def write_sequence_of_pieri_systems():
    """
    Makes a sequence of Pieri systems
    and write each pair of start and target system to file.
    """
    nbrows, nbcols, pvts = ask_inputs()
    answer = raw_input('Double indexing of variables ? (y/n) ')
    linear = (answer != 'y')
    answer = raw_input('Continue through all levels ? (y/n) ')
    contin = (answer == 'y')
    dim = nbrows - nbcols
    planes = random_planes(nbrows, nbcols, pvts)
    start = start_pieri_system(nbrows, nbcols, pvts, planes, linear)
    name = 'pieri' + str(nbrows) + str(nbcols) + 'start0'
    write_to_name_file(name, start, [], nbrows, nbcols)
    newpvts = update_pivots(nbrows, pvts)
    planes.append(random_matrix(nbrows, dim))
    target = pieri_system(nbrows, nbcols, newpvts, planes, linear)
    name = 'pieri' + str(nbrows) + str(nbcols) + 'target0'
    write_to_name_file(name, target, [], nbrows, nbcols)
    cnt = 0
    while True:
        if(not contin):
            answer = raw_input('Continue to next level ? (y/n) ')
            if(answer != 'y'):
                break
        else:
            cnt = cnt + 1
            pvts = [piv for piv in newpvts]
            newpvts = update_pivots(nbrows, pvts)
            if(pvts == newpvts):
                print '-> no extension of pivots possible: no next level'
                break
            start = start_pieri_system(nbrows, nbcols, pvts, planes, linear)
            name = 'pieri' + str(nbrows) + str(nbcols) + 'start' + str(cnt)
            write_to_name_file(name, start, [], nbrows, nbcols)
            planes.append(random_matrix(nbrows, dim))
            target = pieri_system(nbrows, nbcols, newpvts, planes, linear)
            name = 'pieri' + str(nbrows) + str(nbcols) + 'target' + str(cnt)
            write_to_name_file(name, target, [], nbrows, nbcols)

def main():
    """
    Collects the test programs.
    """
    print 'MENU to test setup of Pieri systems :'
    print '  1. show a matrix of variables'
    print '  2. make a Pieri system for pivots'
    print '  3. run path trackers between Pieri systems'
    print '  4. generate sequence of Pieri systems to file'
    answer = raw_input('Type 1, 2, 3, or 4 to select a test : ')
    if(answer == '1'):
        show_variable_matrix()
    elif(answer == '2'):
        show_pieri_system()
    elif(answer == '3'):
        track_sequence_of_pieri_systems()
    elif(answer == '4'):
        write_sequence_of_pieri_systems()
    else:
        print 'Wrong answer, please try again ...'

if __name__ == '__main__':
    main()
